﻿using Excavator.Entities;

namespace Excavator.Drawnings;

public static class ExtentionDrawningTrackedVehicle
{
    /// <summary>
    /// Разделитель для записи информации по объекту в файл
    /// </summary>
    private static readonly string _separatorForObject = ":";
    /// <summary>
    /// Создание объекта из строки
    /// </summary>
    /// <param name="info">Строка с данными для создания объекта</param>
    /// <returns>Объект</returns>
    public static DrawningTrackedVehicle? CreateDrawningTrackedVehicle(this string info)
    {
        string[] strs = info.Split(_separatorForObject);

        EntityTrackedVehicle? car = EntityExcavator.CreateEntityExcavator(strs);
        if (car != null)
        {
            return new DrawningExcavator(car);
        }
        car = EntityTrackedVehicle.CreateEntityTrackedVehicle(strs);
        if (car != null)
        {
            return new DrawningTrackedVehicle(car);
        }
        return null;
    }
    /// <summary>
    /// Получение данных для сохранения в файл
    /// </summary>
    /// <param name="drawningTrackedVehicle">Сохраняемый объект</param>
    /// <returns>Строка с данными по объекту</returns>
    public static string GetDataForSave(this DrawningTrackedVehicle drawningTrackedVehicle)
    {
        string[]? array = drawningTrackedVehicle?.EntityTrackedVehicle?.GetStringRepresentation();
        if (array == null)
        {
            return string.Empty;
        }
        return string.Join(_separatorForObject, array);
    }
}