﻿namespace Excavator.Entities;

/// <summary>
/// Класс-сущность "Экскаватор"
/// </summary>
public class EntityExcavator : EntityTrackedVehicle
{

    /// <summary>
    /// Дополнительный цвет (для опциональных элементов)
    /// </summary>
    public Color AdditionalColor { get; private set; }
    public void SetAdditionalColor(Color additionalColor)
    {
        AdditionalColor = additionalColor;
    }

    /// <summary>
    /// Признак (опция) наличия ковша
    /// </summary>
    public bool Bucket { get; private set; }

    /// <summary>
    /// Признак (опция) наличия опор
    /// </summary>
    public bool Supports { get; private set; }



    /// <summary>
    /// Шаг перемещения экскаватора
    /// </summary>
    public double Step => Speed * 100 / Weight;

    /// <summary>
    /// Инициализация полей объекта-класса экскаватора
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес экскаватора</param>
    /// <param name="bodyColor">Основной цвет</param>
    /// <param name="additionalColor">Дополнительный цвет</param>
    /// <param name="bucket">Признак наличия ковша</param>
    /// <param name="supports">Признак наличия опор</param>

    public EntityExcavator(int speed, double weight, Color bodyColor, Color additionalColor, bool bucket, bool supports) : base(speed, weight, bodyColor)
    {
        AdditionalColor = additionalColor;
        Bucket = bucket;
        Supports = supports;
    }


    /// <summary>
    /// Получение строк со значениями свойств объекта класса-сущности
    /// </summary>
    /// <returns></returns>
    public override string[] GetStringRepresentation()
    {
        return new[] { nameof(EntityExcavator), Speed.ToString(), Weight.ToString(), BodyColor.Name, AdditionalColor.Name, Bucket.ToString(), Supports.ToString() };
    }
    /// <summary>
    /// Создание объекта из массива строк
    /// </summary>
    /// <param name="strs"></param>
    /// <returns></returns>
    public static EntityExcavator? CreateEntityExcavator(string[] strs)
    {
        if (strs.Length != 7 || strs[0] != nameof(EntityExcavator))
        {
            return null;
        }
        return new EntityExcavator(Convert.ToInt32(strs[1]), Convert.ToDouble(strs[2]), Color.FromName(strs[3]),
            Color.FromName(strs[4]), Convert.ToBoolean(strs[5]), Convert.ToBoolean(strs[6]));
    }
}