using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.DependencyInjection;
using Serilog;

namespace Excavator
{
    internal static class Program
    {
        /// <summary>
        ///  The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            ApplicationConfiguration.Initialize();

            ServiceCollection services = new();
            ConfigureServices(services);
            using ServiceProvider serviceProvider = services.BuildServiceProvider();
            Application.Run(serviceProvider.GetRequiredService<FormTrackedVehicleCollection>());
        }

        private static void ConfigureServices(ServiceCollection services)
        {
            services
                .AddSingleton<FormTrackedVehicleCollection>()
                .AddLogging(option =>
                {
                    option.SetMinimumLevel(LogLevel.Information);
                    var config = new ConfigurationBuilder()
                        .AddJsonFile("serilogConfig.json", optional: false, reloadOnChange: true)
                        .Build();
                    option.AddSerilog(Log.Logger = new LoggerConfiguration()
                        .ReadFrom.Configuration(config)
                        .CreateLogger());
                });
        }
    }
}

